

function [b_iso, b_matrix]=calc_b_sLASER(RF,Gx,Gy,Gz)

NI=4; % Number of 180deg pulse

np=length(RF);
dt=6.4e-6; % in seconds
gamma=2.675e8; % in rad/s/T

% Conversion in T/m
Gx=Gx/1000;
Gy=Gy/1000;
Gz=Gz/1000;

G=[Gx,Gy,Gz];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Find position of the centers of the RF pulses %%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% pos_RF is a vector which is equal to 1 when a RF pulse is being executed,
% 0 elsewhere

pos_RF=(RF~=0);

% center_RF is a vector whose length is equal to the number of RF pulses
% and whose components are the instants of RF pulse centers

center_RF=[];
running_max=RF(1);

%%% RF center at RF max

for t=2:np
    if pos_RF(t)==1; % find the max RF amplitude in each pulse
        if RF(t)>running_max;
            running_max=RF(t);
            pos_max=t;
        end
    elseif pos_RF(t-1)==1 % if the pulse has just finished, store the position of the max RF amplitude
        center_RF=[center_RF pos_max];
        running_max=0;
    end
end

%% Possibility to shift the 3 RF pulses

shift_RF=[-250 250 250 250 250];
center_RF=center_RF+shift_RF;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Calculating gradient moments %%%%%%%%%%    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

kx=0*Gx;
ky=0*Gy;
kz=0*Gz;

% Before first quarter echo
t=1;
while prod(center_RF-t)~=0 % this test detects the RF pulses
    kx(t)=0;
    ky(t)=0;
    kz(t)=0;
    t=t+1;
end

%% First 90 and quarter echo
t=t+1;
while prod(center_RF-t)~=0
    kx(t)=kx(t-1)+Gx(t);
    ky(t)=ky(t-1)+Gy(t);
    kz(t)=kz(t-1)+Gz(t);
    t=t+1;
end

% 1st 180
kx(t)=-kx(t-1);
ky(t)=-ky(t-1);
kz(t)=-kz(t-1);
t=t+1;

%% Inter 180 delay

while prod(center_RF-t)~=0
    kx(t)=kx(t-1)+Gx(t);
    ky(t)=ky(t-1)+Gy(t);
    kz(t)=kz(t-1)+Gz(t);
    t=t+1;
end

% 2nd 180
kx(t)=-kx(t-1);
ky(t)=-ky(t-1);
kz(t)=-kz(t-1);
t=t+1;

%% 3rd 180 delay

while prod(center_RF-t)~=0
    kx(t)=kx(t-1)+Gx(t);
    ky(t)=ky(t-1)+Gy(t);
    kz(t)=kz(t-1)+Gz(t);
    t=t+1;
end

% 3rd 180
kx(t)=-kx(t-1);
ky(t)=-ky(t-1);
kz(t)=-kz(t-1);
t=t+1;

%% 4th 180 delay

while prod(center_RF-t)~=0
    kx(t)=kx(t-1)+Gx(t);
    ky(t)=ky(t-1)+Gy(t);
    kz(t)=kz(t-1)+Gz(t);
    t=t+1;
end

% 4th 180
kx(t)=-kx(t-1);
ky(t)=-ky(t-1);
kz(t)=-kz(t-1);
t=t+1;

%% Last delay

while t<np
    kx(t)=kx(t-1)+Gx(t);
    ky(t)=ky(t-1)+Gy(t);  
    kz(t)=kz(t-1)+Gz(t);
    t=t+1;
end

%% Conversion to rad/m
kx=gamma*kx*dt;
ky=gamma*ky*dt;
kz=gamma*kz*dt;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Calculating cross terms and b matrix (s/m) %%    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

kxkx=kx.*kx;
kyky=ky.*ky;
kzkz=kz.*kz;
kxky=kx.*ky;
kxkz=kx.*kz;
kykz=ky.*kz;


bxx=sum(kxkx)*dt
byy=sum(kyky)*dt
bzz=sum(kzkz)*dt
bxy=sum(kxky)*dt
bxz=sum(kxkz)*dt
byz=sum(kykz)*dt

b_matrix=[[bxx bxy bxz];[bxy byy byz];[bxz byz bzz]];

% conversion to s/mm

b_matrix=b_matrix*1e-6

b_iso=trace(b_matrix)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Figures %%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

t=(1:np)*dt*1000;

%%%%%%%%% Chronogram %%%%%%%%%%%

figure;

subplot(4,2,[1 2]);
plot(t,RF,'m');
hold on;
yL = get(gca,'YLim');
for k=1:length(center_RF)
    line([center_RF(k)*dt*1000 center_RF(k)*dt*1000],yL,'LineStyle',':','Color','k');
end
legend('RF pulses');
title('Chronogram');
xlabel('time ms');
ylabel('RF');

subplot(4,2,[3 4]);
plot(t,Gx*1000,'b');
ylim([1.5*min(Gx*1000) 1.5*max(Gx*1000)])
hold on;
yL = get(gca,'YLim');
for k=1:length(center_RF)
    line([center_RF(k)*dt*1000 center_RF(k)*dt*1000],yL,'LineStyle',':','Color','k');
end
legend('Gx')
xlabel('time (ms)');
ylabel('Gx amplitude (mT/m)');

subplot(4,2,[5 6]);
plot(t,Gy*1000,'r');
ylim([1.5*min(Gy*1000) 1.5*max(Gx*1000)]) 
hold on;
yL = get(gca,'YLim');
for k=1:length(center_RF)
    line([center_RF(k)*dt*1000 center_RF(k)*dt*1000],yL,'LineStyle',':','Color','k');
end
legend('Gy')
xlabel('time (ms)');
ylabel('Gy amplitude (mT/m)');

subplot(4,2,[7 8]);
plot(t,Gz*1000,'k');
ylim([1.5*min(Gz*1000) 1.5*max(Gz*1000)])
hold on;
yL = get(gca,'YLim');
for k=1:length(center_RF)
    line([center_RF(k)*dt*1000 center_RF(k)*dt*1000],yL,'LineStyle',':','Color','k');
end
legend('Gz')
xlabel('time (ms)');
ylabel('Gz amplitude (mT/m)');

%%%%%%%%% Gradient moments %%%%%%%%%%%

figure;

subplot(3,2,[1 2]);
plot(t,kx,'b');
ylim([1.5*min(kx) 1.5*max(kx)]) 
hold on;
yL = get(gca,'YLim');
for k=1:length(center_RF)
    line([center_RF(k)*dt*1000 center_RF(k)*dt*1000],yL,'LineStyle',':','Color','k');
end
legend('kx');
title('Gradient moments');
xlabel('time (ms)');

subplot(3,2,[3 4]);
plot(t,ky,'r');
ylim([1.5*min(ky) 1.5*max(ky)])
hold on;
yL = get(gca,'YLim');
for k=1:length(center_RF)
    line([center_RF(k)*dt*1000 center_RF(k)*dt*1000],yL,'LineStyle',':','Color','k');
end
legend('ky');
xlabel('time (ms)');

subplot(3,2,[5 6]);
plot(t,kz,'k');
ylim([1.5*min(kz) 1.5*max(kz)])
hold on;
yL = get(gca,'YLim');
for k=1:length(center_RF)
    line([center_RF(k)*dt*1000 center_RF(k)*dt*1000],yL,'LineStyle',':','Color','k');
end
legend('kz');
xlabel('time (ms)');


%%%%%%%%% Cross-terms %%%%%%%%%%%

figure;

subplot(6,2,[1 2]);
plot(t,kxkx,'b');
ylim([1.5*min(kxkx) 1.5*max(kxkx)]) 
hold on;
yL = get(gca,'YLim');
for k=1:length(center_RF)
    line([center_RF(k)*dt*1000 center_RF(k)*dt*1000],yL,'LineStyle',':','Color','k');
end
legend('kxkx')
title('Cross terms');
xlabel('time (ms)');

subplot(6,2,[3 4]);
plot(t,kyky,'r');
ylim([1.5*min(kyky) 1.5*max(kyky)]) 
hold on;
yL = get(gca,'YLim');
for k=1:length(center_RF)
    line([center_RF(k)*dt*1000 center_RF(k)*dt*1000],yL,'LineStyle',':','Color','k');
end
legend('kyky')
xlabel('time (ms)');

subplot(6,2,[5 6]);
plot(t,kzkz,'k');
ylim([1.5*min(kzkz) 1.5*max(kzkz)]) 
hold on;
yL = get(gca,'YLim');
for k=1:length(center_RF)
    line([center_RF(k)*dt*1000 center_RF(k)*dt*1000],yL,'LineStyle',':','Color','k');
end
legend('kzkz')
xlabel('time (ms)');

subplot(6,2,[7 8]);
plot(t,kxky,'g');
ylim([1.5*min(kxky) 1.5*max(kxky)]) 
hold on;
yL = get(gca,'YLim');
for k=1:length(center_RF)
    line([center_RF(k)*dt*1000 center_RF(k)*dt*1000],yL,'LineStyle',':','Color','k');
end
legend('kxkz')
xlabel('time (ms)');
    
subplot(6,2,[9 10]);
plot(t,kxkz,'g');
ylim([1.5*min(kxkz) 1.5*max(kxkz)]) 
hold on;
yL = get(gca,'YLim');
for k=1:length(center_RF)
    line([center_RF(k)*dt*1000 center_RF(k)*dt*1000],yL,'LineStyle',':','Color','k');
end
legend('kxkz')
xlabel('time (ms)');

subplot(6,2,[11 12]);
plot(t,kykz,'g');
ylim([1.5*min(kykz) 1.5*max(kykz)]) 
hold on;
yL = get(gca,'YLim');
for k=1:length(center_RF)
    line([center_RF(k)*dt*1000 center_RF(k)*dt*1000],yL,'LineStyle',':','Color','k');
end
legend('kykz')
xlabel('time (ms)');